/*
	$Id $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.HashMap;
import java.util.Map;

import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.util.ma.Log10MACalculator;
import net.sf.basedb.util.ma.Log2MACalculator;
import net.sf.basedb.util.ma.MACalculator;


/**
	Gives information in what way spot intensities has been transformed
	before they were stored in the database.
	
	@author Nicklas
	@version 2.12
*/
public enum IntensityTransform 
{
	
	/**
		No transformation has been applied to the spot intensities.
	*/
	NONE(0, "None", false, Formula.AverageMethod.GEOMETRIC_MEAN)
	{
		/**
			@return The same expression
		 */
		@Override
		public Expression unTransform(Expression transformed)
		{
			return transformed;
		}
		/**
			@return The same value
		*/
		@Override
		public double unTransform(double transformed)
		{
			return transformed;
		}
		
		/**
			@return The same expression
		 */
		@Override
		public Expression transform(Expression e)
		{
			return e;
		}
		
		@Override
		public double transform(double value) 
		{
			return value;
		}
		
		/**
			Creates a {@link MACalculator}.
		*/
		@Override
		public MACalculator getMACalculator() 
		{
			return new MACalculator();
		}
	},

	/**
		Spot intensities are stored as base-2 logged values 
	*/
	LOG2(1, "Logarithmic (base-2)", true, Formula.AverageMethod.ARITHMETIC_MEAN)
	{
		
		/**
			@return A new expression: 2 ^ transformed
		 */
		@Override
		public Expression unTransform(Expression transformed)
		{
			return Expressions.power(Expressions.integer(2), transformed);
		}
		/**
			@return 2 raised to the given value
		*/
		@Override
		public double unTransform(double transformed)
		{
			return Math.pow(2.0, transformed);
		}
		
		/**
			@return A new expresion: log2(e)
		*/
		@Override
		public Expression transform(Expression e)
		{
			return Expressions.log2(e);
		}
		
		/**
			@return log2(value)
		 */
		@Override
		public double transform(double value) 
		{
			return Math.log(value) / LN2;
		}
		
		/**
			Creates a {@link Log2MACalculator}.
		*/
		@Override
		public MACalculator getMACalculator() 
		{
			return new Log2MACalculator();
		}
	},
	
	/**
		Spot intensities are stored as base-10 logged values 
	*/
	LOG10(2, "Logarithmic (base-10)", true, Formula.AverageMethod.ARITHMETIC_MEAN)
	{
		/**
			@return A new expression: 10 ^ transformed
		 */
		@Override
		public Expression unTransform(Expression transformed)
		{
			return Expressions.power(Expressions.integer(10), transformed);
		}
		/**
			@return 10 raised to the given value
		*/
		@Override
		public double unTransform(double transformed)
		{
			return Math.pow(10.0, transformed);
		}
		
		/**
			@return A new expresion: log10(e)
		*/
		@Override
		public Expression transform(Expression e)
		{
			return Expressions.log10(e);
		}
		
		/**
			@return log10(value)
		 */
		@Override
		public double transform(double value) 
		{
			return Math.log10(value);
		}
		
		/**
			Creates a {@link Log10MACalculator}.
		*/
		@Override
		public MACalculator getMACalculator() 
		{
			return new Log10MACalculator();
		}
	};

	/**
		Maps an integer to a transform.
	*/
	private static final Map<Integer, IntensityTransform> valueMapping = 
		new HashMap<Integer, IntensityTransform>();

	private static final double LN2 = Math.log(2);
	
	static
	{
		for(IntensityTransform sdt : IntensityTransform.values())
		{
			IntensityTransform t = valueMapping.put(sdt.getValue(), sdt);
			assert t == null : "Another transform with the value "+sdt.getValue()+" already exists";
		}
	}
	
	/**
		The integer value of this transform. 
	*/
	private final int value;
	/**
		Returned by the toString method.
	*/
	private final String displayValue;
	
	/**
		If the intensities are transformed or not.
	*/
	private final boolean isTransformed;
	
	private final Formula.AverageMethod averageMethod;
	
	/**
	 	Creates a new intensity transform. Used internal only.
		@param value The integer value of this location.
	*/
	private IntensityTransform(int value, String displayValue, 
			boolean isTransformed, Formula.AverageMethod averageMethod)
	{
		this.value = value;
		this.displayValue = displayValue;
		this.isTransformed = isTransformed;
		this.averageMethod = averageMethod;
	}
	
	@Override
	public String toString()
	{
		return displayValue;
	}
	
	/**
		Get the <code>SpotDataTransform</code> object when you know the integer code.
		
		@param value The integer value.
		@return The transform for the integer value.
	*/
	public static IntensityTransform fromValue(int value)
	{
		IntensityTransform transform = valueMapping.get(value);
		assert transform != null : "transform == null for value "+value;
		return transform;
	}
	
	/**
		Get the integer value that is used when storing a transform in the database.
		@return The integer value for this transform
	*/
	public int getValue()
	{
		return value;
	}
	
	/**
		Does this intensity transformation represents a transformation that
		has modified the spot intensity values or not?
		@return TRUE if the intensity values are transformed, FALSE if they
			are not
	*/
	public boolean isTransformed()
	{
		return isTransformed;
	}
	
	/**
		Get the averaging method to use when averaging values
		that has been transformed with the specified transformation.
		@return An average method
	*/
	public Formula.AverageMethod getAverageMethod()
	{
		return averageMethod;
	}
	
	/**
		Un-transform an expression representing a transformed intensity
		value so that it represents a raw intensity value. This method
		may return the same expression if no transformation is needed.
		@param transformed The transformed expression
	*/
	public abstract Expression unTransform(Expression transformed);

	/**
		Un-transform a transformed intensity value back to a
		regular intensity value.
		@param transformed The transformed value
	*/
	public abstract double unTransform(double transformed);

	/**
		Transform an expression representing a "normal" intensity value
		so that it represents the transformed intensity value.
		This method	may return the same expression if no transformation 
		is needed.
		@since 3.2.4
		@param e The regular expression
	*/
	public abstract Expression transform(Expression e);
	
	/**
		Transform a "normal" intensity value to the transformed value.
		@param value The regular value
		@since 3.2.4
	*/
	public abstract double transform(double value);
	
	/**
		Get a MA calculator implementation that is suitable for calculating
		MA-values for the given source data type.
		@since 3.2.4
	*/
	public abstract MACalculator getMACalculator();
	
}
