/**
	$Id: AffymetrixUtil.java 5713 2011-09-02 13:01:58Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.affymetrix;

import java.util.List;

import affymetrix.fusion.cdf.FusionCDFData;
import affymetrix.fusion.cel.FusionCELData;
import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileStoreEnabled;
import net.sf.basedb.core.FileStoreUtil;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.InvalidUseOfNullException;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.RawBioAssay;

/**
	Utility class with Affymetrix-related functionality.

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-09-02 15:01:58 +0200 (fr, 02 sep 2011) $
*/
public final class AffymetrixUtil
{

 	/**
		Check if the item is using the affymetrix platform.
	 	@param item Item to check. Null is not allowed.
	 	@return TRUE if platform is used, FALSE otherwise.
	*/
	public static boolean isAffymetrix(FileStoreEnabled item)
	{
		if (item == null) throw new InvalidUseOfNullException("item");
		Platform platform = item.getPlatform();
		if (platform == null || !Platform.AFFYMETRIX.equals(platform.getExternalId()))
		{
			return false;
		}
		return true;
	}

	/**
		Check if the CEL file on the raw bioassay matches the CDF file on the
		array design. If the array design has more than one CDF file only the
		first one is used. If the raw bioassay has more than one CEL file, each
		file will be validated against the CDF.
		
		@param rawBioAssay The raw bioassay
		@param design The array design
		@param required TRUE if the absense of either a CEL or a CDF file is an
			error condition
		@throws InvalidDataException If the raw bioassay or array design are not
			Affymetrix type, or if the CEL doesn't match the CDF
		@since 2.4
	*/
	public static void validateCelAndCdf(RawBioAssay rawBioAssay, ArrayDesign design, boolean required)
	{
		DbControl dc = rawBioAssay.getDbControl();
		List<File> celFiles = FileStoreUtil.getDataFiles(dc, rawBioAssay, DataFileType.AFFYMETRIX_CEL, false);
		File cdfFile = FileStoreUtil.getDataFile(dc, design, DataFileType.AFFYMETRIX_CDF, false);
		if (cdfFile != null && celFiles != null && celFiles.size() > 0)
		{
			String cdfChipType = cdfFile.getName();
			CelValidator celValidator = new CelValidator();
			CdfValidator cdfValidator = new CdfValidator();
			FusionCDFData cdf = cdfValidator.loadCdfFile(cdfFile);
			
			for (File celFile : celFiles)
			{
				FusionCELData cel = celValidator.loadCelFile(celFile);
				celValidator.validateCelAndCdf(cel, cdf, cdfChipType);
			}
		}
		else if (required)
		{
			if (celFiles == null || celFiles.size() == 0)
			{
				throw new InvalidDataException("Raw bioassay has no CEL file: " + rawBioAssay);
			}
			else
			{
				throw new InvalidDataException("Array design has no CDF file: " + design);				
			}
		}
	}

}
