/**
	$Id: SectionReporterListParser.java 5405 2010-09-10 11:09:05Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.importer.spotdata;

import java.io.IOException;
import java.util.List;

import net.sf.basedb.core.AnyToAny;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.core.ReporterBatcher;
import net.sf.basedb.core.ReporterList;
import net.sf.basedb.core.Transformation;
import net.sf.basedb.core.data.ReporterData;
import net.sf.basedb.core.signal.ThreadSignalHandler;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.basefile.BaseFileParser;
import net.sf.basedb.util.basefile.BaseFileSectionParser;
import net.sf.basedb.util.parser.FlatFileParser;

/**
	Parses the 'reporterlist' section of a BASEfile. This section
	is optional. If it exists it must 'reporterId' and 'score'
	data columns in the 'columns' header. The name of the new reporter
	list can be set by the 'name' header. If this doesn't exists a 
	name is genarated automatically.
	<p>
	NOTE! The column names can be redefined by calling
	{@link BaseFileParser#setRedefinedColumnName(String, String, String)}

	@author Nicklas
	@version 2.14
	@base.modified $Date: 2010-09-10 13:09:05 +0200 (fr, 10 sep 2010) $
*/
public class SectionReporterListParser
	implements BaseFileSectionParser
{

	private final DbControl dc;
	private final BaseFileInfo info;
	private final Transformation transformation;
	
	private int sectionCount;
	private final String totalBytes;
	private final long progressReportInterval;
	private long nextProgressReport;

	public SectionReporterListParser(DbControl dc, BaseFileInfo info, Transformation transformation)
	{
		this.dc = dc;
		this.info = info;
		this.transformation = transformation;
		this.progressReportInterval = info.getSize() / 100;
		this.nextProgressReport = progressReportInterval;
		this.totalBytes = Values.formatBytes(info.getSize());
	}
	
	/*
		From the BaseFileSectionParser interface
		-----------------------------------------
	*/
	@Override
	public void parseSection(BaseFileParser parser, FlatFileParser ffp)
		throws IOException
	{
		++sectionCount;
		String filename = info.getName();
		
		// Parse the headers and verify that they meet the requirements
		ffp.parseHeaders();
		int line = ffp.getParsedLines();
		List<String> columns = parser.getRequiredHeader(ffp, "columns", "\\t", "reporterlist", filename);
		int reporterIndex = parser.getRequiredIndex(columns, 
				parser.getRedefinedColumnName("reporterlist", "reporterId"), "columns", "reporterlist", line, filename);
		int scoreIndex = columns.indexOf(parser.getRedefinedColumnName("reporterlist", "score"));
		
		String name = Values.getStringOrNull(ffp.getHeader("name"));
		if (name == null) name = transformation.getName() + "." + sectionCount;
		
		ReporterList list = ReporterList.getNew(dc);
		list.setName(name);
		list.setExperiment(transformation.getExperiment());
		
		FlatFileParser.Data data;
		ReporterBatcher reporters = ReporterBatcher.getNew(dc);

		while ((data = ffp.nextData()) != null)
		{
			ThreadSignalHandler.checkInterrupted();
			long parsedBytes = ffp.getParsedBytes();
			if (parsedBytes >= nextProgressReport)
			{
				nextProgressReport = parsedBytes + progressReportInterval;
				parser.setProgress(parsedBytes, "Importing reporter list '" + name + "': " + 
						Values.formatBytes(parsedBytes) + " of " + totalBytes);
			}
			
			String reporterId = Values.getStringOrNull(data.get(reporterIndex));
			// Ignore missing reporters
			if (reporterId == null) continue;
			
			ReporterData reporter = reporters.getByExternalId(reporterId, false);
			if (reporter == null)
			{
				throw new ItemNotFoundException("Reporter[externalId=" + reporterId + 
						"] in section 'reporterlist' at line " +
						ffp.getParsedLines() + " in file '" +filename + "'");
			}
			// If no score is provided we use the line number as some kind of 'rank'
			Float score = scoreIndex == -1 ? 
				data.dataLineNo() : Values.getFloat(data.get(scoreIndex), null);
			list.addReporter(reporter, score);
		}

		dc.saveItem(list);
		AnyToAny any2Any = AnyToAny.getNew(dc, transformation, list, "reporterList." + sectionCount, false);
		dc.saveItem(any2Any);
	}
	// -----------------------------------------
}
