/**
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.listable;

import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**
	Source item transformer implementation that transform extract items to their
	parent extract items. From the given set of extract ID:s the transformer will
	recursively load as many parent extracts as possible. The source items
	may optionally be included in the target set.

	@author Nicklas
	@since 3.5
*/
public class ExtractToParentExtractTransformer 
	extends AbstractSourceItemTransformer
{

	private final boolean includeSourcesInTarget;
	private CollectExtracts collectedExtracts;
	
	/**
		Create a new extract to parent extract transformer.
		@param includeSourcesInTarget TRUE to include the source items in the target
	*/
	public ExtractToParentExtractTransformer(boolean includeSourcesInTarget) 
	{
		super(Item.EXTRACT, Item.EXTRACT);
		this.includeSourcesInTarget = includeSourcesInTarget;
	}
	
	/**
		Create a new extract to parent extract transformer that is considering
		extracts collected on downstream derived and raw bioassays.
		@param includeSourcesInTarget TRUE to include the source items in the target
		@param collectedExtracts Information about extracts found on downstream derived and raw bioassays
	 */
	public ExtractToParentExtractTransformer(boolean includeSourcesInTarget, CollectExtracts collectedExtracts) 
	{
		this(includeSourcesInTarget);
		this.collectedExtracts = collectedExtracts;
	}
	
	@Override
	public Set<Integer> transform(TransformContext context, Set<Integer> source) 
	{
		DbControl dc = context.getDbControl();
		
		ItemQuery<Extract> query = Extract.getQuery();
		query.setIncludes(context.getInclude());
		query.join(Hql.innerJoin("childCreationEvents", "ce"));
		query.join(Hql.innerJoin("ce", "event", "evt"));
		query.join(Hql.innerJoin("evt", "bioMaterial", "bm"));
		query.restrict(
			Restrictions.in(
				Hql.alias("bm"), 
				Expressions.parameter("children")
			)
		);

		// If we have collected downstream extracts we must
		// ADD ALL parent extracts to those that were collected
		// REMOVE ALL extracts that are not found among the children to the collected extracts
		Set<Integer> upstream = null;
		Set<Integer> downstream = null;		
		if (collectedExtracts != null)
		{
			// The 'upstream' is all collected extracts and their parents
			upstream = collectedExtracts.getCollectedParents();
			// The 'downstream' is all collected and their children
			downstream = collectedExtracts.getCollectedChildren();

			// We remove all upstream since those are already loaded
			// and only keep those that are found downstream
			source = new HashSet<Integer>(source);
			source.removeAll(upstream);
			source.retainAll(downstream);
		}
		
		// Keep track of all seen children and parents
		Set<Integer> allParents = new HashSet<Integer>();
		Set<Integer> allChildren = new HashSet<Integer>();
		if (includeSourcesInTarget) allParents.addAll(source);

		// Children that we have not yet checked
		Set<Integer> children = source;
		
		while (children.size() > 0)
		{
			Set<Integer> parents = safeIdList(dc, query, "children", children);
			//query.setParameter("children", children, Type.INT);
			//Set<Integer> parents = new HashSet<Integer>(query.idList(dc));
			
			if (upstream != null) parents.removeAll(upstream);
			if (downstream != null) parents.retainAll(downstream);
			
			// Store new parents and "used" children
			allParents.addAll(parents);
			allChildren.addAll(children);
			
			// Remove children that we have already seen...
			// ...the remaining parents become our new children
			parents.removeAll(allChildren);
			children = parents;
		}
		
		// Finally add all upstream parent extracts
		if (upstream != null) allParents.addAll(upstream);
		
		return allParents;
	}


}
