import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.ParameterType;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PlatformVariant;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginConfigurationRequest;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.PluginResponse;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.plugin.Response;

/*
	$Id $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

public class TestGtfImporters
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing GTF importers using plugin");
		// Upload GTF file
		int fileId = TestFile.test_create("data/test.gtf", false, false);
		
		// Test reporter importer
		int gtfReporterImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.gtf.GtfReporterImporter");
		ok = gtfReporterImporterId != 0;
		int reporterJobId = test_create_reporter_job(gtfReporterImporterId, fileId, "update");
		ok &= TestJob.test_execute(reporterJobId, false);
		TestReporter.write_header();
		TestReporter.test_list(35);
		

		// Test reporter map importer
		int arrayDesignId = TestArrayDesign.test_create(PlatformVariant.SEQUENCING_EXPRESSION, false);
		int gtfFeatureImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.gtf.GtfReporterMapImporter");
		ok = gtfFeatureImporterId != 0;
		int featureConfigurationId = test_create_feature_configuration(gtfFeatureImporterId);
		int featureJobId = test_create_feature_job(featureConfigurationId, fileId, arrayDesignId);
		ok &= TestJob.test_execute(featureJobId, false);
		TestArrayDesign.write_feature_header();
		TestArrayDesign.test_list_features(arrayDesignId, 35);

		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		TestArrayDesign.test_delete(arrayDesignId);
		// Delete reporters
		int deleteReporterJobId = test_create_reporter_job(gtfReporterImporterId, fileId, "delete");
		ok &= TestJob.test_execute(deleteReporterJobId, false);
		
		TestJob.test_delete(reporterJobId);
		TestJob.test_delete(deleteReporterJobId);
		TestJob.test_delete(featureJobId);
		TestPluginConfiguration.test_delete(featureConfigurationId);
		TestFile.test_delete(fileId);

		write("++Testing GTF importers "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}


	static int test_create_reporter_job(int pluginId, int fileId, String mode)
	{
		if (pluginId == 0 || fileId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.JOB)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginDefinition plugin = PluginDefinition.getById(dc, pluginId);
			Job j = Job.getNew(dc, plugin, null, null);
			j.setName("Test: " + plugin.getName());
			
			PluginConfigurationRequest request = j.configure(new GuiContext(Item.REPORTER, GuiContext.Type.LIST));
			write_request_information(request.getRequestInformation());
			for (PluginParameter<?> pp : request.getRequestInformation().getParameters())
			{
				Object value = request.getCurrentParameterValue(pp.getName());
				if (value == null) value = pp.getDefaultValue();
				if (value != null) request.setParameterValue(pp.getName(), value);
			}
			request.setParameterValue("file", File.getById(dc, fileId));
			request.setParameterValue("mode", mode);
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(j);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = j.getId();
			write("--Create job for GTF reporter import (mode="+mode+") OK");
		}
		catch (Throwable ex)
		{
			write("--Create job for GTF reporter import (mode="+mode+") FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}


	static void write_request_information(RequestInformation ri)
	{
		if (TestUtil.getSilent()) return;

		write("Request information for command: "+ri.getCommand());
		write("\tTitle:\t"+ri.getTitle());
		write("\tDescription:\t"+ri.getDescription());
		for (PluginParameter<?> pp : ri.getParameters())
		{
			ParameterType<?> pt = pp.getParameterType();
			System.out.println("\tParameter:\t"+pp.getName() + "\t" + pp.getLabel() + "\t" + pt + "\t" + pp.getDefaultValue() );
		}
	}
	
	static int test_create_feature_configuration(int pluginId)
	{
		if (pluginId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.PLUGINCONFIGURATION)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginDefinition pd = PluginDefinition.getById(dc, pluginId);
			PluginConfiguration pc = pd.newPluginConfiguration();
			pc.setName(pd.getName());
			
			PluginConfigurationRequest request = pc.configure();
			RequestInformation ri = request.getRequestInformation();

			write_request_information(ri);
			
			for (PluginParameter<?> parameter : ri.getParameters())
			{
				request.setParameterValue(parameter.getName(), parameter.getDefaultValue());
			}
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(pc);
				dc.commit();
			}
			else if (response.getStatus() == Response.Status.ERROR)
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			else
			{
				throw new BaseException("Unknown status on response: "+response.getStatus());
			}
			id = pc.getId();
			write("--Create configuration for GTF reporter map import OK");
		}
		catch (Throwable ex)
		{
			write("--Create configuration for GTF reporter map import FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static int test_create_feature_job(int pluginConfigurationId, int fileId, int arrayDesignId)
	{
		if (pluginConfigurationId == 0 || fileId == 0 || arrayDesignId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.JOB)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginConfiguration pc = PluginConfiguration.getById(dc, pluginConfigurationId);
			Job j = pc.newJob(null);
			j.setName(pc.getName());
			
			PluginConfigurationRequest request = j.configure(null);
			write_request_information(request.getRequestInformation());
			request.setParameterValue("file", File.getById(dc, fileId));
			request.setParameterValue("arrayDesign", ArrayDesign.getById(dc, arrayDesignId));
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(j);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = j.getId();
			write("--Create job for GTF reporter map import OK");
		}
		catch (Throwable ex)
		{
			write("--Create job for GTF reporter map import FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

}

